#ifdef USE_LIGHTS

#define LIGHT_OMNI 0
#define LIGHT_OMNINORMAL 1
#define LIGHT_SPOT 2
#define LIGHT_SPOTNORMAL 3
#define LIGHT_SHADOW 4

// lights
BeginBlock(Lights)
_uniform int numLights;
_uniform vec4 lightParams[3 * MAX_LIGHTS];
EndBlock

#ifdef SHADOW_BOX
// shadow box
BeginBlock(ShadowBox)
_uniform int numShadows;
_uniform vec4 shadowParams[5 * MAX_SHADOWS];
EndBlock
#endif  // SHADOW_BOX

void ProcessLight(int i)
{
  vec4 pos = lightParams[3 * i];
  vec4 params = lightParams[3 * i + 1];
  vec4 dir = lightParams[3 * i + 2];

  vec3 delta = pos.xyz - inPosition.xyz;
  float reach = pos.w;
  float dist = dot(delta, delta);
  if (dist < reach) {
    vec3 color = params.xyz;
    int type = int(params.w);
    float scale = 1.0 - (dist / reach);
    dist = sqrt(dist);

    // calc angle from normal if needed
    if (type == LIGHT_OMNINORMAL || type == LIGHT_SPOTNORMAL) {
      float ang = dot(delta, inNormal);
      ang = clamp(ang / dist, 0.0, 1.0);
      scale *= ang;
    }

    // calc cone adjustment if needed
    if (type == LIGHT_SPOT || type == LIGHT_SPOTNORMAL) {
      float cone = -dot(dir.xyz, delta) / dist - 1.0;
      float conemul = dir.w;
      cone = cone * conemul + 1.0;
      cone = clamp(cone, 0.0, 1.0);
      scale *= cone;
    }

    varColor.rgb += mix(vec3(0.0), color, scale);
  }
}

#ifdef SHADOW_BOX
int ProcessShadow(int i)
{
  vec4 pos = shadowParams[5 * i];
  vec4 params = shadowParams[5 * i + 1];
  mat3 dir = mat3(
      shadowParams[5 * i + 2].xyz,
      shadowParams[5 * i + 3].xyz,
      shadowParams[5 * i + 4].xyz);

  vec3 delta = pos.xyz - inPosition.xyz;
  vec3 dist = delta * dir;
  vec3 size = params.xyz;
  return int(
      all(greaterThan(dist, -size)) && 
      all(lessThan(dist, size)));
}
#endif  // SHADOW_BOX

void ProcessLights(void)
{
  for (int i = 0; i < MAX_LIGHTS; ++i) {
    if (i < numLights) {
      ProcessLight(i);
    } else {
      break;
    }
  }

  #ifdef SHADOW_BOX
  int shadowFlag = 0;
  for (int i = 0; i < MAX_SHADOWS; ++i) {
    if (i < numShadows) {
      shadowFlag += ProcessShadow(i);
    } else {
      break;
    }
  }
  if (shadowFlag > 0) {
    varColor.rgb += shadowColor;
  }
  #endif  // SHADOW_BOX
}
#endif  // USE_LIGHTS
